/**
 * @author Hamadou BA
 * Dépôt GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp6-ps-methode-des-moindres-carrees.git
 * 
 * Classe responsable de la visualisation des données et du modèle polynomial.
 * Utilise la bibliothèque XChart pour afficher graphiquement les points de support
 * et la courbe d'approximation polynomiale.
 */

package tp6.ps;

import org.knowm.xchart.XYChart;
import org.knowm.xchart.XYChartBuilder;
import org.knowm.xchart.XYSeries;
import org.knowm.xchart.style.Styler;
import org.knowm.xchart.style.markers.SeriesMarkers;
import org.knowm.xchart.SwingWrapper;
import org.knowm.xchart.BitmapEncoder;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;

public class ChartVisualizer {
    
  
    private static final String CAPTURES_FOLDER = "captures";
    
  
    public static void visualizePolynomial(ModPoly poly, double[] x, double[] y, String filename) {

    	XYChart chart = new XYChartBuilder()
            .width(800)
            .height(600)
            .title("Approximation par moindres carrés")
            .xAxisTitle("X")
            .yAxisTitle("Y")
            .build();
        
        chart.getStyler().setLegendPosition(Styler.LegendPosition.InsideNE);
        chart.getStyler().setDefaultSeriesRenderStyle(XYSeries.XYSeriesRenderStyle.Scatter);
        chart.getStyler().setMarkerSize(10);
        
        XYSeries dataSeries = chart.addSeries("Points de support", x, y);
        dataSeries.setMarker(SeriesMarkers.CIRCLE);
        
        int numPoints = 100;
        double xMin = Arrays.stream(x).min().getAsDouble();
        double xMax = Arrays.stream(x).max().getAsDouble();
        double step = (xMax - xMin) / (numPoints - 1);
        
        double[] xCurve = new double[numPoints];
        double[] yCurve = new double[numPoints];
        
        for (int i = 0; i < numPoints; i++) {
            xCurve[i] = xMin + i * step;
            yCurve[i] = poly.evaluate(xCurve[i]);
        }
        
        XYSeries polySeries = chart.addSeries("Polynôme (degré " + poly.getDegree() + ")", xCurve, yCurve);
        polySeries.setXYSeriesRenderStyle(XYSeries.XYSeriesRenderStyle.Line);
        polySeries.setMarker(SeriesMarkers.NONE);
        
        new SwingWrapper<>(chart).displayChart();
        
        if (filename != null) {
            try {

            	Path capturesDirPath = Paths.get("target", "classes", CAPTURES_FOLDER);
                if (!Files.exists(capturesDirPath)) {
                    Files.createDirectories(capturesDirPath);
                }
                
                String fullPath = capturesDirPath.toString() + File.separator + filename;
                
                BitmapEncoder.saveBitmap(chart, fullPath, BitmapEncoder.BitmapFormat.PNG);
                
                System.out.println("Graphique sauvegardé dans le fichier " + fullPath + ".png");
                
            } catch (IOException e) {
                System.err.println("Erreur lors de la sauvegarde du graphique: " + e.getMessage());
            }
        }
    }
}