
/**
 * @author Hamadou BA
 * Dépôt GitLab : https://www-apps.univ-lehavre.fr/forge/bh243413/tp6-ps-methode-des-moindres-carrees.git
 * 
 * Classe ModPoly pour l'approximation polynomiale par moindres carrés.
 * Cette classe permet de construire un modèle polynomial qui approxime
 * au mieux un ensemble de points selon le critère des moindres carrés.
 * Les fonctions de base sont définies par fi(x) = x^i.
 */

package tp6.ps;

public class ModPoly {
	
    private double[] coefficients;
    private int degree;

    
    public ModPoly(int degree) {
        this.degree = degree;
        this.coefficients = new double[degree + 1];
    }

   
    public double evaluate(double x) {
        double result = 0.0;
        for (int i = 0; i <= degree; i++) {
            result += coefficients[i] * Math.pow(x, i);
        }
        return result;
    }

  
    private double baseFunction(int i, double x) {
        return Math.pow(x, i);
    }

  
    public void identifie(double[] x, double[] y) {
        int n = x.length;
        int m = degree + 1; 
        
        double[][] A = new double[m][m];
        double[] b = new double[m];
        
        for (int i = 0; i < m; i++) {
            for (int j = 0; j < m; j++) {
                double sum = 0.0;
                for (int k = 0; k < n; k++) {
                    sum += baseFunction(i, x[k]) * baseFunction(j, x[k]);
                }
                A[i][j] = sum;
            }
            
            double sum = 0.0;
            for (int k = 0; k < n; k++) {
                sum += y[k] * baseFunction(i, x[k]);
            }
            b[i] = sum;
        }
        
        coefficients = solveSystem(A, b);
    }
    
    /**
     * Résout un système d'équations linéaires Ax = b en utilisant l'élimination de Gauss
     * @param A Matrice des coefficients
     * @param b Vecteur des termes constants
     * @return Vecteur solution
     */
    
    private double[] solveSystem(double[][] A, double[] b) {
        int n = b.length;
        
        double[][] augmented = new double[n][n + 1];
        for (int i = 0; i < n; i++) {
            System.arraycopy(A[i], 0, augmented[i], 0, n);
            augmented[i][n] = b[i];
        }
        
        for (int i = 0; i < n; i++) {

        	int maxRow = i;
            for (int j = i + 1; j < n; j++) {
                if (Math.abs(augmented[j][i]) > Math.abs(augmented[maxRow][i])) {
                    maxRow = j;
                }
            }
            
            if (maxRow != i) {
                double[] temp = augmented[i];
                augmented[i] = augmented[maxRow];
                augmented[maxRow] = temp;
            }
            
            
            for (int j = i + 1; j < n; j++) {
                double factor = augmented[j][i] / augmented[i][i];
                for (int k = i; k <= n; k++) {
                    augmented[j][k] -= factor * augmented[i][k];
                }
            }
        }
        
        double[] solution = new double[n];
        for (int i = n - 1; i >= 0; i--) {
            solution[i] = augmented[i][n];
            for (int j = i + 1; j < n; j++) {
                solution[i] -= augmented[i][j] * solution[j];
            }
            solution[i] /= augmented[i][i];
        }
        
        return solution;
    }
    
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("p(x) = ");
        
        for (int i = 0; i <= degree; i++) {
            if (i > 0) {
                if (coefficients[i] >= 0) {
                    sb.append(" + ");
                } else {
                    sb.append(" - ");
                }
                sb.append(String.format("%.4f", Math.abs(coefficients[i])));
                if (i == 1) {
                    sb.append("x");
                } else {
                    sb.append("x^").append(i);
                }
            } else {
                sb.append(String.format("%.4f", coefficients[i]));
            }
        }
        
        return sb.toString();
    }
    
   
    public double[] getCoefficients() {
        return coefficients;
    }
    
 
    public int getDegree() {
        return degree;
    }
}